;/*****************************************************************************/
;/* STARTUP.S: Startup file for Hello Example                                 */
;/*****************************************************************************/
;/* <<< Use Configuration Wizard in Context Menu >>>                          */ 
;/*****************************************************************************/
;/* This file is part of the uVision/ARM development tools.                   */
;/* Copyright (c) 2005-2006 Keil Software. All rights reserved.               */
;/* This software may only be used under the terms of a valid, current,       */
;/* end user licence from KEIL for a compatible version of KEIL software      */
;/* development tools. Nothing else gives you the right to use this software. */
;/*****************************************************************************/


; *** Startup Code (executed after Reset) ***


; Standard definitions of Mode bits and Interrupt (I & F) flags in PSRs

Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UND        EQU     0x1B
Mode_SYS        EQU     0x1F

I_Bit           EQU     0x80            ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40            ; when F bit is set, FIQ is disabled


; Internal RAM Address
RAM_Base        EQU     0x00000000      ; after Ramap
RAM_Base_Boot   EQU     0x00300000      ; after Reset until Remap 


;// <h> Stack Configuration (Stack Sizes in Bytes)
;//   <o0> Undefined Mode      <0x0-0xFFFFFFFF:8>
;//   <o1> Supervisor Mode     <0x0-0xFFFFFFFF:8>
;//   <o2> Abort Mode          <0x0-0xFFFFFFFF:8>
;//   <o3> Fast Interrupt Mode <0x0-0xFFFFFFFF:8>
;//   <o4> Interrupt Mode      <0x0-0xFFFFFFFF:8>
;//   <o5> User/System Mode    <0x0-0xFFFFFFFF:8>
;// </h>

UND_Stack_Size  EQU     0x00000000
SVC_Stack_Size  EQU     0x00000100
ABT_Stack_Size  EQU     0x00000000
FIQ_Stack_Size  EQU     0x00000000
IRQ_Stack_Size  EQU     0x00000080
USR_Stack_Size  EQU     0x00000400

ISR_Stack_Size  EQU     (UND_Stack_Size + SVC_Stack_Size + ABT_Stack_Size + \
                         FIQ_Stack_Size + IRQ_Stack_Size)

                AREA    STACK, NOINIT, READWRITE, ALIGN=3

Stack_Mem       SPACE   USR_Stack_Size
__initial_sp    SPACE   ISR_Stack_Size
Stack_Top


;// <h> Heap Configuration
;//   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF>
;// </h>

Heap_Size       EQU     0x00000000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit


;// <h> External Bus Interface (EBI)
EBI_BASE        EQU     0xFFE00000

;//   <e.13> Enable Chip Select 0 (CSR0)
;//     <o.20..31> BA: Base Address <0x0-0xFFF00000:0x100000><#/0x100000>
;//                <i> Start Address for Chip Select Signal
;//     <o.7..8>   PAGES: Page Size      <0=> 1M Byte    <1=> 4M Bytes
;//                                      <2=> 16M Bytes  <3=> 64M Bytes
;//                <i> Selects Active Bits in Base Address
;//     <o.0..1>   DBW: Data Bus Width   <1=> 16-bit     <2=> 8-bit
;//     <o.12>     BAT: Byte Access Type <0=> Byte-write
;//                                      <1=> Byte-select
;//     <e.5>      WSE: Enable Wait State Generation
;//       <o.2..4>   NWS: Number of Standard Wait States <1-8><#-1>
;//     </e>
;//     <o.9..11>  TDF: Data Float Output Time <0-7>
;//                <i> Number of Cycles Added after the Transfer
;//   </e>
EBI_CSR0_Val    EQU     0x0100252d

;//   <e.13> Enable Chip Select 1 (CSR1)
;//     <o.20..31> BA: Base Address <0x0-0xFFF00000:0x100000><#/0x100000>
;//                <i> Start Address for Chip Select Signal
;//     <o.7..8>   PAGES: Page Size      <0=> 1M Byte    <1=> 4M Bytes
;//                                      <2=> 16M Bytes  <3=> 64M Bytes
;//                <i> Selects Active Bits in Base Address
;//     <o.0..1>   DBW: Data Bus Width   <1=> 16-bit     <2=> 8-bit
;//     <o.12>     BAT: Byte Access Type <0=> Byte-write
;//                                      <1=> Byte-select
;//     <e.5>      WSE: Enable Wait State Generation
;//       <o.2..4>   NWS: Number of Standard Wait States <1-8><#-1>
;//     </e>
;//     <o.9..11>  TDF: Data Float Output Time <0-7>
;//                <i> Number of Cycles Added after the Transfer
;//   </e>
EBI_CSR1_Val    EQU     0x02002121

;//   <e.13> Enable Chip Select 2 (CSR2)
;//     <o.20..31> BA: Base Address <0x0-0xFFF00000:0x100000><#/0x100000>
;//                <i> Start Address for Chip Select Signal
;//     <o.7..8>   PAGES: Page Size      <0=> 1M Byte    <1=> 4M Bytes
;//                                      <2=> 16M Bytes  <3=> 64M Bytes
;//                <i> Selects Active Bits in Base Address
;//     <o.0..1>   DBW: Data Bus Width   <1=> 16-bit     <2=> 8-bit
;//     <o.12>     BAT: Byte Access Type <0=> Byte-write
;//                                      <1=> Byte-select
;//     <e.5>      WSE: Enable Wait State Generation
;//       <o.2..4>   NWS: Number of Standard Wait States <1-8><#-1>
;//     </e>
;//     <o.9..11>  TDF: Data Float Output Time <0-7>
;//                <i> Number of Cycles Added after the Transfer
;//   </e>
EBI_CSR2_Val    EQU     0x20000000

;//   <e.13> Enable Chip Select 3 (CSR3)
;//     <o.20..31> BA: Base Address <0x0-0xFFF00000:0x100000><#/0x100000>
;//                <i> Start Address for Chip Select Signal
;//     <o.7..8>   PAGES: Page Size      <0=> 1M Byte    <1=> 4M Bytes
;//                                      <2=> 16M Bytes  <3=> 64M Bytes
;//                <i> Selects Active Bits in Base Address
;//     <o.0..1>   DBW: Data Bus Width   <1=> 16-bit     <2=> 8-bit
;//     <o.12>     BAT: Byte Access Type <0=> Byte-write
;//                                      <1=> Byte-select
;//     <e.5>      WSE: Enable Wait State Generation
;//       <o.2..4>   NWS: Number of Standard Wait States <1-8><#-1>
;//     </e>
;//     <o.9..11>  TDF: Data Float Output Time <0-7>
;//                <i> Number of Cycles Added after the Transfer
;//   </e>
EBI_CSR3_Val    EQU     0x30000000

;//   <e.13> Enable Chip Select 4 (CSR4)
;//     <o.20..31> BA: Base Address <0x0-0xFFF00000:0x100000><#/0x100000>
;//                <i> Start Address for Chip Select Signal
;//     <o.7..8>   PAGES: Page Size      <0=> 1M Byte    <1=> 4M Bytes
;//                                      <2=> 16M Bytes  <3=> 64M Bytes
;//                <i> Selects Active Bits in Base Address
;//     <o.0..1>   DBW: Data Bus Width   <1=> 16-bit     <2=> 8-bit
;//     <o.12>     BAT: Byte Access Type <0=> Byte-write
;//                                      <1=> Byte-select
;//     <e.5>      WSE: Enable Wait State Generation
;//       <o.2..4>   NWS: Number of Standard Wait States <1-8><#-1>
;//     </e>
;//     <o.9..11>  TDF: Data Float Output Time <0-7>
;//                <i> Number of Cycles Added after the Transfer
;//   </e>
EBI_CSR4_Val    EQU     0x40000000

;//   <e.13> Enable Chip Select 5 (CSR5)
;//     <o.20..31> BA: Base Address <0x0-0xFFF00000:0x100000><#/0x100000>
;//                <i> Start Address for Chip Select Signal
;//     <o.7..8>   PAGES: Page Size      <0=> 1M Byte    <1=> 4M Bytes
;//                                      <2=> 16M Bytes  <3=> 64M Bytes
;//                <i> Selects Active Bits in Base Address
;//     <o.0..1>   DBW: Data Bus Width   <1=> 16-bit     <2=> 8-bit
;//     <o.12>     BAT: Byte Access Type <0=> Byte-write
;//                                      <1=> Byte-select
;//     <e.5>      WSE: Enable Wait State Generation
;//       <o.2..4>   NWS: Number of Standard Wait States <1-8><#-1>
;//     </e>
;//     <o.9..11>  TDF: Data Float Output Time <0-7>
;//                <i> Number of Cycles Added after the Transfer
;//   </e>
EBI_CSR5_Val    EQU     0x50000000

;//   <e.13> Enable Chip Select 6 (CSR6)
;//     <o.20..31> BA: Base Address <0x0-0xFFF00000:0x100000><#/0x100000>
;//                <i> Start Address for Chip Select Signal
;//     <o.7..8>   PAGES: Page Size      <0=> 1M Byte    <1=> 4M Bytes
;//                                      <2=> 16M Bytes  <3=> 64M Bytes
;//                <i> Selects Active Bits in Base Address
;//     <o.0..1>   DBW: Data Bus Width   <1=> 16-bit     <2=> 8-bit
;//     <o.12>     BAT: Byte Access Type <0=> Byte-write
;//                                      <1=> Byte-select
;//     <e.5>      WSE: Enable Wait State Generation
;//       <o.2..4>   NWS: Number of Standard Wait States <1-8><#-1>
;//     </e>
;//     <o.9..11>  TDF: Data Float Output Time <0-7>
;//                <i> Number of Cycles Added after the Transfer
;//   </e>
EBI_CSR6_Val    EQU     0x60000000

;//   <e.13> Enable Chip Select 7 (CSR7)
;//     <o.20..31> BA: Base Address <0x0-0xFFF00000:0x100000><#/0x100000>
;//                <i> Start Address for Chip Select Signal
;//     <o.7..8>   PAGES: Page Size      <0=> 1M Byte    <1=> 4M Bytes
;//                                      <2=> 16M Bytes  <3=> 64M Bytes
;//                <i> Selects Active Bits in Base Address
;//     <o.0..1>   DBW: Data Bus Width   <1=> 16-bit     <2=> 8-bit
;//     <o.12>     BAT: Byte Access Type <0=> Byte-write
;//                                      <1=> Byte-select
;//     <e.5>      WSE: Enable Wait State Generation
;//       <o.2..4>   NWS: Number of Standard Wait States <1-8><#-1>
;//     </e>
;//     <o.9..11>  TDF: Data Float Output Time <0-7>
;//                <i> Number of Cycles Added after the Transfer
;//   </e>
EBI_CSR7_Val    EQU     0x70000000

;//   <o.0..2>     ALE: Address Line Enable
;//                     <0=> 16MB (CS0..3)
;//                     <1=> 16MB (CS0..3)
;//                     <2=> 16MB (CS0..3)
;//                     <3=> 16MB (CS0..3)
;//                     <4=> 8MB (CS0..4)
;//                     <5=> 4MB (CS0..5)
;//                     <6=> 2MB (CS0..6)
;//                     <7=> 1MB (CS0..7)
;//   <q.4>        DRP: Data Read Protocol
;//                     <0=> Standard Read
;//                     <1=> Early Read
EBI_MCR_Val     EQU     0x00000006

;// </h> End of EBI


; Area Definition and Entry Point
; Startup Code must be linked first at Address at which it expects to run.

        PRESERVE8        

        AREA    RESET, CODE, READONLY
        ARM
        IMPORT  IRQ_Handler

; Exception Vectors (before Remap)
;  Mapped to Address 0 before Remap Command.
;  Relative addressing mode must be used.

Reset_Vec       B       Reset_Handler
Undef_Vec       B       Undef_Vec
SWI_Vec         B       SWI_Vec
PAbt_Vec        B       PAbt_Vec
DAbt_Vec        B       DAbt_Vec
                NOP
IRQ_Vec         B       IRQ_Vec
FIQ_Vec         B       FIQ_Vec


; Exception Vectors (after Remap)
;  Mapped to Address 0 after Remap Command.
;  Absolute addressing mode must be used.
;  Dummy Handlers are implemented as infinite loops which can be modified.

Vectors         LDR     PC, Reset_Addr         
                LDR     PC, Undef_Addr
                LDR     PC, SWI_Addr
                LDR     PC, PAbt_Addr
                LDR     PC, DAbt_Addr
                NOP                            ; Reserved Vector 
                LDR     PC, IRQ_Addr
                LDR     PC, FIQ_Addr
                LDR     PC, [PC, #-0xF20]      ; Vector from AIC_FVR 

Reset_Addr      DCD     Soft_Reset
Undef_Addr      DCD     Undef_Handler
SWI_Addr        DCD     SWI_Handler
PAbt_Addr       DCD     PAbt_Handler
DAbt_Addr       DCD     DAbt_Handler
                DCD     0                      ; Reserved Address 
IRQ_Addr        DCD     IRQ_Handler
FIQ_Addr        DCD     FIQ_Handler

Soft_Reset      B       Soft_Reset
Undef_Handler   B       Undef_Handler
SWI_Handler     B       SWI_Handler
PAbt_Handler    B       PAbt_Handler
DAbt_Handler    B       DAbt_Handler
FIQ_Handler     B       FIQ_Handler

; Reset Handler

                EXPORT  Reset_Handler
Reset_Handler   


; Copy Exception Vectors to Internal RAM before Remap

                ADR     R8, Vectors
                IF      :DEF:REMAPPED
                MOV     R9, #RAM_Base
                ELSE
                MOV     R9, #RAM_Base_Boot
                ENDIF
                LDMIA   R8!, {R0-R7}           ; Load Vectors 
                STMIA   R9!, {R0-R7}           ; Store Vectors 
                LDMIA   R8!, {R0-R7}           ; Load Handler Addresses 
                STMIA   R9!, {R0-R7}           ; Store Handler Addresses 

                IF      :DEF:REMAPPED          ; Execute remapping only, if not already done!
                B       AfterRemap
				ENDIF

; Initialise EBI and execute Remap

                LDR     R12, AfterRemapAdr     ; Get the Real Jump Address
				ADR     R11, EBI_Table 
                LDMIA   R11, {R0-R10}          ; Load EBI Data 
                STMIA   R10, {R0-R9}           ; Store EBI Data with Remap 
                BX      R12                    ; Jump and flush Pipeline 

EBI_Table       DCD     EBI_CSR0_Val
                DCD     EBI_CSR1_Val
                DCD     EBI_CSR2_Val
                DCD     EBI_CSR3_Val
                DCD     EBI_CSR4_Val
                DCD     EBI_CSR5_Val
                DCD     EBI_CSR6_Val
                DCD     EBI_CSR7_Val
                DCD     0x00000001             ; Remap Command 
                DCD     EBI_MCR_Val
                DCD     EBI_BASE

AfterRemapAdr   DCD     AfterRemap

AfterRemap


; Initialise Interrupt System
;  ...

; --- Initialize stack pointer registers

; Enter modes and setup stacks


        IMPORT  ||Image$$CSTACK$$ZI$$Limit||       ; defined in scatter file
        IMPORT  ||Image$$IRQ_STACK$$ZI$$Limit||    ; defined in scatter file

        IF :DEF:FASTVERSION
; More compact, but produces warnings
        msr     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit ; No interrupts
        ldr     sp, =||Image$$IRQ_STACK$$ZI$$Limit||
        and     sp, sp, #0xFFFFFFF8

; Define and initialize other stacks if required here         

; Ensure, main is entered in SVC mode!
; Do not switch to any other mode. Do NOT enter USER mode!

        msr     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts
        ldr     sp, =||Image$$CSTACK$$ZI$$Limit||
        and     sp, sp, #0xFFFFFFF8

        ELSE
; Slower version which assembles without warnings
        msr     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit ; No interrupts
        ldr     r0, =||Image$$IRQ_STACK$$ZI$$Limit||
        and     r0, r0, #0xFFFFFFF8
		mov     sp, r0

; Define and initialize other stacks if required here         

; Ensure, main is entered in SVC mode!
; Do not switch to any other mode. Do NOT enter USER mode!

        msr     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts
        ldr     r0, =||Image$$CSTACK$$ZI$$Limit||
        and     r0, r0, #0xFFFFFFF8
		mov     sp, r0

		ENDIF

        IMPORT  __low_level_init
        LDR     R0, =__low_level_init
        mov     lr, pc
		BX      R0
       
; Enter the C code

        IMPORT  __main
        LDR     R0, =__main
        BX      R0


	    END
