/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2014  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.04a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for AT91SAM9G45-EK
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/

#define PIOA_ID          (2)
#define PIOB_ID          (3)
#define PIOC_ID          (4)
#define PIODE_ID         (5)  // Parallel I/O Controller D and E

#define PIOA_BASE_ADDR   (0xfffff200)
#define PIOB_BASE_ADDR   (0xfffff400)
#define PIOC_BASE_ADDR   (0xfffff600)
#define PIOD_BASE_ADDR   (0xFFFFF800)
#define PIOE_BASE_ADDR   (0xFFFFFA00)

#define PIO_PER_OFFS     (0x00)
#define PIO_OER_OFFS     (0x10)
#define PIO_SODR_OFFS    (0x30)       // Set output data
#define PIO_CODR_OFFS    (0x34)       // Clear output data register
#define PIO_ODSR_OFFS    (0x38)       // output data status register
#define PIO_PDSR_OFFS    (0x3c)       // pin data status register

//
// Map port for EMAC, AT91SAM9G45 has EMAC connected to port A
//
#define EMAC_PORT_ID          (PIOA_ID)
#define EMAC_PORT_BASE_ADDR   (PIOA_BASE_ADDR)
#define EMAC_PORT_PPUDR       (*(volatile unsigned int*) (EMAC_PORT_BASE_ADDR + 0x60)) // Pull up disable register
#define EMAC_PORT_PPUER       (*(volatile unsigned int*) (EMAC_PORT_BASE_ADDR + 0x64)) // Pull up enable register
#define EMAC_PORT_ASR         (*(volatile unsigned int*) (EMAC_PORT_BASE_ADDR + 0x70)) // Select peripheral function A register
#define EMAC_PORT_BSR         (*(volatile unsigned int*) (EMAC_PORT_BASE_ADDR + 0x74)) // Select peripheral function B register
#define EMAC_PORT_PDR         (*(volatile unsigned int*) (EMAC_PORT_BASE_ADDR + 0x04)) // PIO disable register

#define PIOC_PORT_BSR         (*(volatile unsigned int*) (PIOC_BASE_ADDR + 0x74))      // Select peripheral function B register
#define PIOC_PORT_PPUDR       (*(volatile unsigned int*) (PIOC_BASE_ADDR + 0x60))      // Pull up disable register
#define PIOC_PORT_PDR         (*(volatile unsigned int*) (PIOC_BASE_ADDR + 0x04))      // PIO disable register

#define EMAC_PORT_RXDV_BIT    (15)
#define EMAC_PORT_RMII_BIT    (30)

/****** SFRs and identifiers used for LED demo **********************/
#define PMC_BASE_ADDR         (0xFFFFFC00)
#define PMC_PCER              (*(volatile unsigned int*) (PMC_BASE_ADDR + 0x10))

/****** Assign LEDs to Ports ****************************************/

/****** Target specific configuration *******************************/

#define LEDPORT0_BASE_ADDR    (PIOD_BASE_ADDR)
#define LEDPORT1_BASE_ADDR    (PIOD_BASE_ADDR)

#define LEDPORT0_ID           (PIODE_ID)
#define LEDPORT1_ID           (PIODE_ID)

#define LED0_BIT              ( 0) // LED D6 on AT91SAM9G45-EK board, active low
#define LED1_BIT              (31) // LED D7 on AT91SAM9G45-EK board, active low

#define LEDPORT0_PER          (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_PER_OFFS))
#define LEDPORT0_OER          (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_OER_OFFS))
#define LEDPORT0_SODR         (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_SODR_OFFS))
#define LEDPORT0_CODR         (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_CODR_OFFS))
#define LEDPORT0_STATE        (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_ODSR_OFFS))

#define LEDPORT1_PER          (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_PER_OFFS))
#define LEDPORT1_OER          (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_OER_OFFS))
#define LEDPORT1_SODR         (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_SODR_OFFS))
#define LEDPORT1_CODR         (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_CODR_OFFS))
#define LEDPORT1_STATE        (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_ODSR_OFFS))

#define LEDMASK0_ALL          (1uL << LED0_BIT)
#define LEDMASK1_ALL          (1uL << LED1_BIT)

/*********************************************************************
*
*       Static const
*
**********************************************************************
*/
typedef struct {
  volatile unsigned int * pSODR;
  volatile unsigned int * pCODR;
  volatile unsigned int * pSTATE;
  char                    Id;
} LED_TYPE;

static const LED_TYPE _aLED[] = {
  {&LEDPORT0_SODR, &LEDPORT0_CODR, &LEDPORT0_STATE, LED0_BIT},
  {&LEDPORT1_SODR, &LEDPORT1_CODR, &LEDPORT1_STATE, LED1_BIT}
};

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  PMC_PCER     |= (1uL << LEDPORT0_ID) /* Enable peripheral clock for LED-Ports    */
                | (1uL << LEDPORT1_ID)
                ;
  LEDPORT0_PER   = LEDMASK0_ALL;       /* Enable Port           */
  LEDPORT0_OER   = LEDMASK0_ALL;       /* Switch to output mode */
  LEDPORT0_SODR |= LEDMASK0_ALL;       /* Initially clear LEDs  */
  LEDPORT1_PER   = LEDMASK1_ALL;       /* Enable Port           */
  LEDPORT1_OER   = LEDMASK1_ALL;       /* Switch to output mode */
  LEDPORT1_SODR |= LEDMASK1_ALL;       /* Initially clear LEDs  */
}


/*********************************************************************
*
*       BSP_ETH_Init()
*
*  Function description
*    This function is called from the network interface driver.
*    It initializes the network interface. This function should be used
*    to enable the ports which are connected to the network hardware.
*    It is called from the driver during the initialization process.
*
*  Note:
*    (1) If your MAC is connected to the PHY via Media Independent
*        Interface (MII) change the macro _USE_RMII and call
*        IP_NI_ConfigPHYMode() from within IP_X_Config()
*        to change the default of driver.
*
*/
void BSP_ETH_Init(unsigned Unit) {
  unsigned int PinsA_EMAC;
  unsigned int PinsB_EMAC;

  (void) Unit;    // Multiple units not supported by this version, suppress warning

  PMC_PCER          = (1uL << EMAC_PORT_ID);        // Enable clock for PIO
  EMAC_PORT_PPUDR   = (1uL << EMAC_PORT_RXDV_BIT);  // Disable RXDV pullup, enter PHY normal mode
  //
  // Init PIO Pins: EMAC is connected to specific lines of PIO
  //
  PinsA_EMAC        = (1uL << 10)   // ETX0
                    | (1uL << 11)   // ETX1
                    | (1uL << 12)   // ERX0
                    | (1uL << 13)   // ERX1
                    | (1uL << 14)   // ETXEN
                    | (1uL << 15)   // ERXDV
                    | (1uL << 16)   // ERXER
                    | (1uL << 17)   // ETXCK
                    | (1uL << 18)   // EMDC
                    | (1uL << 19)   // EMDIO
                    ;
  PinsB_EMAC        = 0;            // Not used for RMII mode (which is default)
#if 1
   //
   // By default driver uses RMII. Note that IP_NI_ConfigPHYMode() has also
   // to called from within IP_X_Config to switch to MII mode.
   //
  EMAC_PORT_PPUER   = (1uL << EMAC_PORT_RMII_BIT);  // Enable Pullup  => Switch to RMII mode.
#else
  EMAC_PORT_PPUDR   = (1uL << EMAC_PORT_RMII_BIT);  // Disable Pullup => Switch to MII mode.
  PinsB_EMAC       |= (1uL << 6)    // ETX2
                    | (1uL << 7)    // ETX3
                    | (1uL << 8)    // ERX2
                    | (1uL << 9)    // ERx3
                    | (1uL << 27)   // ETXER
                    | (1uL << 28)   // ERXCK
                    | (1uL << 29)   // ECRS
                    | (1uL << 30)   // ECOL
                    ;

#endif
  //
  // Power up PHY, may not be required, if set as hardwired option on target
  //
#ifdef EMAC_PORT_PWR_PHY_BIT
  EMAC_PORT_PER  = (1uL << EMAC_PORT_PWR_PHY_BIT);
  EMAC_PORT_OER  = (1uL << EMAC_PORT_PWR_PHY_BIT);
  EMAC_PORT_CODR = (1uL << EMAC_PORT_PWR_PHY_BIT);
#endif

  EMAC_PORT_ASR = PinsA_EMAC;   // Select peripheral A use on EMAC port
  EMAC_PORT_BSR = PinsB_EMAC;   // Select peripheral B use on EMAC port
  EMAC_PORT_PDR = PinsA_EMAC;   // Disable GPIO mode, select peripheral function
  EMAC_PORT_PDR = PinsB_EMAC;   // Disable GPIO mode, select peripheral function
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  if(Index > 1) {
    return;
  }
  *_aLED[Index].pCODR = (1uL << _aLED[Index].Id);    /* Switch on LED  */
}

void BSP_ClrLED(int Index) {
  if(Index > 1) {
    return;
  }
  *_aLED[Index].pSODR = (1uL << _aLED[Index].Id);    /* Switch off LED */
}

void BSP_ToggleLED(int Index) {
  if(Index > 1) {
    return;
  }
  if (*_aLED[Index].pSTATE & (1 << _aLED[Index].Id)) {  /* LED is switched off */
    BSP_SetLED(Index);
  } else {
    BSP_ClrLED(Index);
  }
}

/*********************************************************************
*
*       BSP_CACHE_CleanRange()
*/
void BSP_CACHE_CleanRange (void * p, unsigned NumBytes) {
  OS_ARM_DCACHE_CleanRange(p, NumBytes);
}

/*********************************************************************
*
*       BSP_CACHE_InvalidateRange()
*/
void BSP_CACHE_InvalidateRange(void * p, unsigned NumBytes) {
  OS_ARM_DCACHE_InvalidateRange(p, NumBytes);
}

/****** End of file *************************************************/

