/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2014  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.04a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Maxim MAX32590 Evaulation Kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define GPIO0_BASE_ADDR     (0xFFE08000u)
#define GPIO0_EN            (*(volatile unsigned int*) (GPIO0_BASE_ADDR + 0x0000u))      // GPIO Function Enable Register 
#define GPIO0_OUT_EN        (*(volatile unsigned int*) (GPIO0_BASE_ADDR + 0x000Cu))      // GPIO Output Enable Register 
#define GPIO0_OUT           (*(volatile unsigned int*) (GPIO0_BASE_ADDR + 0x0018u))      // GPIO Output Register 
#define GPIO0_OUT_SET       (*(volatile unsigned int*) (GPIO0_BASE_ADDR + 0x001Cu))      // GPIO Output Set 
#define GPIO0_OUT_CLR       (*(volatile unsigned int*) (GPIO0_BASE_ADDR + 0x0020u))      // GPIO Output Clear 

#define LEDHEARTBEAT        (1u << 0u) // LED Heartbeat GPIO0.0 D9

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  GPIO0_OUT_EN |= LEDHEARTBEAT;
  GPIO0_OUT_SET = LEDHEARTBEAT;
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    GPIO0_OUT_CLR = LEDHEARTBEAT;
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    GPIO0_OUT_SET = LEDHEARTBEAT;
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if ((GPIO0_OUT & LEDHEARTBEAT) == 0x00u) {
      GPIO0_OUT_SET = LEDHEARTBEAT;
    } else {
      GPIO0_OUT_CLR = LEDHEARTBEAT;
    }
  }
}

/*********************************************************************
*
*       BSP_CACHE_CleanRange()
*/
void BSP_CACHE_CleanRange (void * p, unsigned NumBytes) {
  OS_ARM_DCACHE_CleanRange(p, NumBytes);
}

/*********************************************************************
*
*       BSP_CACHE_InvalidateRange()
*/
void BSP_CACHE_InvalidateRange(void * p, unsigned NumBytes) {
  OS_ARM_DCACHE_InvalidateRange(p, NumBytes);
}

/****** End Of File *************************************************/
