/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2014  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.04a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_MAX32590.c for Maxim MAX32590 CPU

Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS.
          Feel free to modify this file acc. to your
          target system.

--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#define ALLOW_NESTED_INTERRUPTS 0   // Caution: Nesting interrupts will cause higher stack load on system stack CSTACK

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS  
  #define OS_FSYS (384000000)       
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER (OS_FSYS/4) // APB Peripheral Clock 
#endif

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

#ifndef   OS_VIEW_USE_UART          // If set, UART will be used
  #define OS_VIEW_USE_UART  (0)     // Default: 0 => DCC is used
#endif                              // if OS_VIEW_ENABLE is on

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently Uart0 is supported.
*       When using DCC for communication, the UART is not used for embOSView,
*       regardless the OS_UART settings
*/
#ifndef   OS_UART
  #define OS_UART (-1)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART (OS_FSYS/2)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

/****** End of configuration settings *******************************/

/*********************************************************************
*
*       DCC and UART settings for OSView
*
*       Automatically generated from configuration settings
*/
#define OS_USE_DCC     (OS_VIEW_ENABLE && (OS_VIEW_USE_UART == 0))

#define OS_UART_USED   (OS_VIEW_ENABLE && (OS_VIEW_USE_UART != 0)) && ((OS_UART == 0))

/********************************************************************/

#if (DEBUG || OS_USE_DCC)
  #include "JLINKDCC.h"
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define TIMER0_BASE_ADDR       0xFFE18000u 
#define TIMER0_CNT             (*(volatile unsigned int*) (TIMER0_BASE_ADDR + 0x0000u))      // Timer Count register
#define TIMER0_CMP             (*(volatile unsigned int*) (TIMER0_BASE_ADDR + 0x0004u))      // Timer Compare register
#define TIMER0_PWM             (*(volatile unsigned int*) (TIMER0_BASE_ADDR + 0x0008u))      // Timer PWM register
#define TIMER0_INT             (*(volatile unsigned int*) (TIMER0_BASE_ADDR + 0x000Cu))      // Timer Int register
#define TIMER0_CN              (*(volatile unsigned int*) (TIMER0_BASE_ADDR + 0x0010u))      // Timer Control register 
#define TIMER_CONTINUOUS       (1u << 0u)
#define TIMER_DIV1             (0u << 3u)
#define TIMER_ENABLE           (1u << 7u)
#define TIMER_IRQ_CLR          (1u << 0u)
#define OS_TIMER_RELOAD        (OS_PCLK_TIMER/OS_TICK_FREQ)

#define INTC_BASE_ADDR         0xFFE03000u
#define INTC_EN                (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0000u))        // Interrupt Enable Register 0 
#define INTC_EN_SET0           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0004u))        // Interrupt Enable Set Register 0 
#define INTC_EN_CLR0           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0008u))        // Interrupt Enable Clear Register 0 
#define INTC_EN1               (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x000Cu))        // Interrupt Enable Register 1 
#define INTC_EN_SET1           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0010u))        // Interrupt Enable Set Register 1 
#define INTC_EN_CLR1           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0014u))        // Interrupt Enable Clear Register 1 
#define INTC_DFLT_VECTR        (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0018u))        // Default Vector 
#define INTC_INT_STAT0         (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0038u))        // Interrupt Status Register 0 
#define INTC_INT_STAT1         (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x003Cu))        // Interrupt Status Register 1 
#define INTC_RAW_INT_STAT0     (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0040u))        // Raw Interrupt Status Register 0 
#define INTC_RAW_INT_STAT1     (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0044u))        // Raw Interrupt Status Register 1 
#define INTC_IRQ_DBG           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0048u))        // IRQ Processor Debug Register 
#define INTC_FIQ_DBG           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x004Cu))        // FIQ Processor Debug Register 
#define INTC_VEC_ADDR           ((volatile unsigned int*) (INTC_BASE_ADDR + 0x0100u))        // Channel Vector Register base address
#define INTC_CFG_ADDR           ((volatile unsigned int*) (INTC_BASE_ADDR + 0x0200u))        // Channel Configuration Register base address
#define INTC_IRQ_VEC           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0F00u))        // IRQ Vector Register 
#define INTC_FIQ_VEC           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0F04u))        // FIQ Vector Register 
#define INTC_IRQ_END           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0F08u))        // IRQ End-of-Interrupt Register 
#define INTC_FIQ_END           (*(volatile unsigned int*) (INTC_BASE_ADDR + 0x0F0Cu))        // FIQ End-of-Interrupt Register 
#define NUM_INT_SOURCES        64u
#define INTC_TIMER0_ID         8u
#define INTC_TIMER0_PRIO       0x07u    // Lowest priority
#define INTC_PRIORITY_MASK     0x07u
#define VECTOR_TABLE_SIZE      100u

#define SRAM_START_ADDR        0x00900000u

/*********************************************************************
*
*       External data
*
**********************************************************************
*/
extern OS_U32 __program_start;
extern OS_U32 __vector_end;

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       MMU and cache configuration
*
* NOTES:
*   The translation table has to be aligned on 16384 bytes boundary
*   The translation table has to be located in un-initialized memory,
*   because it may be filled from startup, before segments and variables
*   are initialized.
*/
#if defined __CC_ARM
  #pragma arm section zidata = "NO_INIT"         //   Has to be declared as separate section UNINIT in the scatter file
  __align(16384) static unsigned int _TranslationTable[0x1000];
  #pragma arm section zidata // back to default (.bss section)
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _Systick_Handler()
*
* Function description 
*   embOS system tick interrupt handler
*/
static void _Systick_Handler(void) {
  TIMER0_INT = TIMER_IRQ_CLR; 
  OS_TICK_Handle(); 
#if (DEBUG || OS_USE_DCC)
  DCC_Process();
#endif
}

/*********************************************************************
*
*       _DefaultIrqHandler()
*
* Function description 
*   This handler is initially written into all AIC interrupt vectors
*   It is called, if no interrupt vector was installed for
*   specific interrupt source.
*   May be used during debugging to detect uninstalled interrupts
*/
static void _DefaultIrqHandler(void) {
  while (1);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
* Function description 
*   Initialize the hardware (timer) required for the OS to run.
*   May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_U32 i;
  
  OS_IncDI();
  OS_ARM_CACHE_Sync();          // Ensure, caches are synchronized
  //
  // Init INTC interrupt controller
  // 
  INTC_DFLT_VECTR = (OS_U32)_DefaultIrqHandler; 
  for (i = 0 ; i < NUM_INT_SOURCES; i++) {
    INTC_VEC_ADDR[i] = (OS_U32)_DefaultIrqHandler;
    INTC_CFG_ADDR[i] = 0x07; // Lowest prio and irq
  }
  //
  // Init timer
  // 
  TIMER0_CNT = 0x00;
  TIMER0_CMP = OS_TIMER_RELOAD;
  TIMER0_CN  = TIMER_ENABLE | TIMER_CONTINUOUS | TIMER_DIV1; 
  OS_ARM_InstallISRHandler(INTC_TIMER0_ID, _Systick_Handler);
  OS_ARM_ISRSetPrio(INTC_TIMER0_ID, INTC_TIMER0_PRIO);
  OS_ARM_EnableISR(INTC_TIMER0_ID);
  OS_COM_Init();
  OS_DecRI();
}

/*********************************************************************
*
*       (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {  /* Idle loop: No task is ready to exec */
  while (1) {
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via OSView or high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = OS_GetTime32();
  Cnt  = TIMER0_CNT;
  //
  // Check if timer interrupt pending ...
  //
  if ((INTC_INT_STAT0 & (1u << INTC_TIMER0_ID)) != 0x00u) {          
    Cnt = TIMER0_CNT;     // Interrupt pending, re-read timer and adjust result
    Time++;
  }
  return ((OS_PCLK_TIMER/1000u) * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/

#if OS_UART_USED

#define UART0_BASE_ADDR        0xFFE28000u
#define UART0_CTRL             (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x0000))       // UART Control Register 
#define UART0_STAT             (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x0004))       // UART Status Register
#define UART0_INT_EN           (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x0008))       // UART Interrupt Enable Register 
#define UART0_INT_STAT         (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x000C))       // UART Interrupt Status Register 
#define UART0_BAUD0            (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x0010))       // UART Baud Rate Register 0 
#define UART0_BAUD1            (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x0014))       // UART Baud Rate Register 1 
#define UART0_DATA             (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x0020))       // UART Data Buffer Register 
#define UART_BAUDRATE_IDIV     (OS_PCLK_UART / (128u * OS_BAUDRATE))                        // IDIV = Nominal Frequency / (128 * Baud Rate Frequency)
#define UART_BAUDRATE_DDIV     (OS_PCLK_UART / (128u * OS_BAUDRATE))                        // DDIV =  (DIV  IDIV) * 128 
#define UART_FFRXIE            (1u << 4u)
#define UART_FFTXHIE           (1u << 6u)

#define UART_FFRXIS            (1u << 4u)
#define UART_FFTXHIS           (1u << 6u)

#define UART_RXEMPTY           (1u << 4u)

#define INTC_UART0_ID          16u
#define INTC_UART0_PRIO        0x07u

#define GPIO0_BASE_ADDR        (0xFFE08000u)
#define GPIO0_EN               (*(volatile unsigned int*) (GPIO0_BASE_ADDR + 0x0000u))      // GPIO Function Enable Register 
#define GPIO0_EN1              (*(volatile unsigned int*) (GPIO0_BASE_ADDR + 0x0068u))      // GPIO Alternate Function Enable Register 
#define PIN_UART0_RXD          18u                                                          // GPIO0.18
#define PIN_UART0_TXD          19u                                                          // GPIO0.19


/*********************************************************************
*
*       OS_COM_ISR_Uart() OS USART interrupt handler
*       handles both, Rx and Tx interrupt
*/
static void _OS_COM_ISR_Uart(void) {
  OS_U32 c;
  //OS_U32 uart_stat;
  OS_U32 uart_int_stat;
  
  //uart_stat     = UART0_STAT;
  uart_int_stat = UART0_INT_STAT;
  //if ((uart_stat & UART_RXEMPTY) == 0u) {
  if ((uart_int_stat & UART_FFRXIS) == UART_FFRXIS) {
    UART0_INT_STAT &= ~UART_FFRXIS; 
    c = UART0_DATA;
    OS_OnRx(c);
  } 
  if ((uart_int_stat & UART_FFTXHIS) == UART_FFTXHIS) {
    UART0_INT_STAT &= ~UART_FFTXHIS;      
    if (OS_OnTx()) {                     // No more characters to send?  
      UART0_INT_EN &= ~UART_FFTXHIE;     // Disable further tx interrupts
    }   
  }
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  //UART0_INT_EN |= UART_FFTXHIE;  // Enable tx interrupt
  UART0_INT_EN |= (1u << 5u);
  //UART0_INT_EN |= (1u << 6u);
  
  //if (((UART0_STAT & 0xF000u) >> 12u) < 8u) {
    UART0_DATA = c;
  //} 
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
void OS_COM_Init(void) {
  OS_IncDI();
  GPIO0_EN     &= ~((1u << PIN_UART0_RXD) | (1u << PIN_UART0_TXD)); // Set to alternate function
  GPIO0_EN1    &= ~((1u << PIN_UART0_RXD) | (1u << PIN_UART0_TXD)); // Set to secondary alternate function
  UART0_CTRL    = 0x00000F01; // 8N1, flush tx and rx buffer
  UART0_BAUD0   = UART_BAUDRATE_IDIV; //0x0000000d;
  UART0_INT_EN |= UART_FFRXIE;
  OS_ARM_InstallISRHandler(INTC_UART0_ID, _OS_COM_ISR_Uart);
  OS_ARM_ISRSetPrio(INTC_UART0_ID, INTC_UART0_PRIO);
  OS_ARM_EnableISR(INTC_UART0_ID);
  OS_DecRI();
}

#elif OS_USE_DCC

/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  (void) Channel;   // Not supported by this version, suppress warning
  OS_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  (void) Channel;   // Not supported by this version, suppress warning
  OS_OnTx();
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize DCC for OSView
*/
void OS_COM_Init(void) {
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Send 1 character via DCC
*/
void OS_COM_Send1(OS_U8 c) {
  JLINKDCC_SendCharOnChannelNB(1, c);
}

#else  /* DCC not selected, selected UART not supported, using dummies */

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* avoid compiler warning */
  OS_COM_ClearTxActive();  /* let the OS know that Tx is not busy */
}

#endif /*  OS_UART_USED  */

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from OS_IRQ_SERVICE function
*       found in RTOSVect.asm
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;

  pISR = (OS_ISR_HANDLER*)INTC_IRQ_VEC; // Get active vector address
  
#if ALLOW_NESTED_INTERRUPTS
  OS_EnterNestableInterrupt();          // Now interrupts may be reenabled. If nesting should be allowed
#else
  OS_EnterInterrupt();                  // Inform OS that interrupt handler is running
#endif
  pISR();                               // Call interrupt service routine
  OS_DI();                              // Disable interrupts and unlock
  
#if ALLOW_NESTED_INTERRUPTS
  OS_LeaveNestableInterrupt();          // Leave nestable interrupt, perform task switch if required
#else
  OS_LeaveInterrupt();                  // Leave interrupt, perform task switch if required
#endif
  INTC_IRQ_END = 0x00;                  // Acknowledge end of interrupt
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler (int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if DEBUG
  if ((unsigned)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return NULL;
  }
#endif
  OS_DI();
  papISR = (OS_ISR_HANDLER**)INTC_VEC_ADDR;
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_RestoreI();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if DEBUG
  if ((unsigned)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif
  OS_DI();
  if (ISRIndex <= 31) {
    INTC_EN_SET0 = (1u << ISRIndex);
  } else {
    INTC_EN_SET1 = (1u << ISRIndex);
  }     
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_DisableISR
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if DEBUG
  if ((unsigned)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif
  OS_DI();
  if (ISRIndex <= 31) {
    INTC_EN_CLR0 = (1u << ISRIndex);
  } else {
    INTC_EN_CLR1 = (1u << ISRIndex);
  }     
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;

#if DEBUG
  if ((unsigned)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return 0;
  }
#endif
  OS_DI();
  pPrio = (OS_U32*)INTC_CFG_ADDR;
  OldPrio = pPrio[ISRIndex];
  pPrio[ISRIndex] = (OldPrio & ~INTC_PRIORITY_MASK) | (Prio & INTC_PRIORITY_MASK);
  OS_RestoreI();
  return (OldPrio & INTC_PRIORITY_MASK);
}

/*********************************************************************
*
*       __low_level_init()
*
*       Initialize memory controller, clock generation and pll
*
*       Has to be modified, if another CPU clock frequency should be used.
*       This function is called during startup and
*       has to return 1 to perform segment initialization
*       Because __low_level_init() is called before segment initialization,
*       no access that relies on any variable can be performed.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no ptototype" warning
#ifdef __cplusplus
  }
#endif
OS_INTERWORK int __low_level_init(void) {
  OS_U32 i;
  OS_U32 *pSrc; 
  OS_U32 *pDest;
  OS_U32 Len;
  //
  // Copy vector table from 0x00920000 to 0x00900000 in order to have vectors at 0x00 after MMU init
  //
  pSrc  = (OS_U32*)&__program_start;
  pDest = (OS_U32*)SRAM_START_ADDR;
  Len   = ((OS_U32)&__vector_end - (OS_U32)&__program_start) >> 2;
  for (i = 0; i < Len; i++) {
    *pDest = *pSrc;
    pDest++;
    pSrc++;
  }
  //
  // Init MMU and caches. This defines the virtual memory map, which is used during execution.
  // Memory mapping should be complete, meaning 4096 entries.
  // Code below fills in ascending VAddr order
  //
  //                                             Mode                      VAddr  PAddr  Size[MB]
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x000, 0x009, 0x001);    // internal SRAM, remap 1MB to address 0x00 to in order place interrupt vectors at 0x00
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x001, 0x001, 0x008);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x009, 0x009, 0x001);    // Internal SRAM, original address, cacheable, bufferable
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x00A, 0x00A, 0x001);    // NVSRAM0 
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x00B, 0x00B, 0x0F5);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x100, 0x100, 0x200);    // External NAND
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x300, 0x300, 0x040);    // Static memory nCS0 (Controller 0)
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x340, 0x340, 0x040);    // Static memory nCS1 (Controller 0)
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x380, 0x380, 0x040);    // Static memory nCS2 (Controller 0)
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x3C0, 0x3C0, 0x040);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x400, 0x400, 0x040);    // Static memory nCS4 (Controller 1)
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x440, 0x440, 0x007);    // Static memory nCS5 (Controller 1) PSRAM (6 MByte)
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0x447, 0x447, 0x039);    // Static memory nCS5 (Controller 1) 
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_NB,    0x480, 0x480, 0x040);    // Static memory nCS6 (Controller 1)
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x4C0, 0x4C0, 0x040);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x500, 0x500, 0x200);    // SDRAM/DDR (Controller 0)
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x700, 0x700, 0x200);    // SDRAM/DDR (Controller 1)
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x900, 0x900, 0x6FE);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0xFFE, 0xFFE, 0x002);    // SFR area NON cacheable, NON bufferable
  OS_ARM_MMU_Enable      (&_TranslationTable[0]);

  OS_ARM_ICACHE_Enable();
  OS_ARM_DCACHE_Enable();

  return 1;
}

/*****  End Of File  ************************************************/
