;/****************************************************************

; Standard definitions of Mode bits and Interrupt (I & F) flags in PSRs



Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UND        EQU     0x1B
Mode_SYS        EQU     0x1F

I_Bit           EQU     0x80            ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40            ; when F bit is set, FIQ is disabled


;----------------------- Memory Definitions ------------------------------------

; Internal Memory Base Addresses
IRAM_BASE       EQU     0x0930000
IROM_BASE       EQU     0x0920000



;----------------------- Stack and Heap Definitions ----------------------------

;// <h> Stack Configuration (Stack Sizes in Bytes)
;//   <o0> Undefined Mode      <0x0-0xFFFFFFFF:8>
;//   <o1> Supervisor Mode     <0x0-0xFFFFFFFF:8>
;//   <o2> Abort Mode          <0x0-0xFFFFFFFF:8>
;//   <o3> Fast Interrupt Mode <0x0-0xFFFFFFFF:8>
;//   <o4> Interrupt Mode      <0x0-0xFFFFFFFF:8>
;//   <o5> User/System Mode    <0x0-0xFFFFFFFF:8>
;// </h>

UND_Stack_Size  EQU     0x00000000
SVC_Stack_Size  EQU     0x00000008
ABT_Stack_Size  EQU     0x00000000
FIQ_Stack_Size  EQU     0x00000000
IRQ_Stack_Size  EQU     0x00000080
USR_Stack_Size  EQU     0x00000400

ISR_Stack_Size  EQU     (UND_Stack_Size + SVC_Stack_Size + ABT_Stack_Size + \
                         FIQ_Stack_Size + IRQ_Stack_Size)

                AREA    STACK, NOINIT, READWRITE, ALIGN=3

Stack_Mem       SPACE   USR_Stack_Size
__initial_sp    SPACE   ISR_Stack_Size
Stack_Top


;// <h> Heap Configuration
;//   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF>
;// </h>

Heap_Size       EQU     0x4000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit

CONFIG_SYS_CPU_CLOCK	EQU 384000000
CONFIG_SYS_CLOCK		EQU (CONFIG_SYS_CPU_CLOCK/4)
PCK_SETUP 				EQU 0x31

SZ_64M              	EQU 0x04000000
SZ_128M             	EQU 0x08000000



MAXIMASP_GCR_PHYS		EQU 0xffe01000
MAXIMASP_WDT_PHYS		EQU 0xffe04000
MAXIMASP_DMC_PHYS		EQU 0xfff5ac00
MAXIMASP_INTC_PHYS		EQU 0xffe03000
MAXIMASP_SIR_PHYS		EQU 0xffe00000
MAXIMASP_NFC_PHYS		EQU 0xfff5a800
MAXIMASP_UCI_PHYS		EQU 0xfff5a000
MAXIMASP_UCI_0_PHYS		EQU (MAXIMASP_UCI_PHYS)
MAXIMASP_UCI_1_PHYS		EQU (MAXIMASP_UCI_PHYS + 0x1000)

MAXIMASP_NAND			EQU 0x10000000	; Shares UCI0 with Primary 

MAXIMASP_DDR0			EQU 0x70000000	; Primary Bus    (UCI0)    


WDT_CTRL				EQU 0x00
GCR_SCON				EQU 0x00	; System Control Register
GCR_SCON_TAP			EQU (1<<0)
GCR_SCON_ARB_SHIFT		EQU 1
GCR_SCON_ARB_MASK		EQU (0x3 << GCR_SCON_ARB_SHIFT)
GCR_SCON_ARB_FIXED		EQU 0  ; Fixed priority AHB master arbitration 
GCR_SCON_ARB_RROBIN		EQU 1  ; Round-robin AHB master arbitration 
GCR_SCON_ARB_PROG		EQU 2  ; Programmable AHB master arbitration 
GCR_SCON_SYNC			EQU (1<<18)
GCR_SCON_ADK			EQU (1<<28)
GCR_SCON_PBEN			EQU (1<<29)
GCR_SCON_PBFB			EQU (1<<30)
GCR_SCON_UBP			EQU (1<<31)
GCR_RSTR				EQU 0x04	; Reset Register 

GCR_CLKCN				EQU 0x08	; Clock Control Register 
GCR_CLKCN_HDIV_SHIFT	EQU 0
GCR_CLKCN_HDIV_MASK		EQU 0x07
GCR_CLKCN_PDIV_SHIFT	EQU 3
GCR_CLKCN_PDIV_MASK		EQU 0x38
GCR_CLKCN_PSC_SHIFT		EQU 6
GCR_CLKCN_PSC_MASK		EQU 0x1c0
GCR_CLKCN_CLKSEL_SHIFT	EQU 9
GCR_CLKCN_CLKSEL_MASK	EQU 0xe00
GCR_CLKCN_CKRDY			EQU (1<<13)

GCR_PM					EQU 0x0c	; Power Management Register 
GCR_PLLCN				EQU 0x14
GCR_PLL_EN				EQU (1<<0)	; PLL Enable 
GCR_PLL_RDY				EQU (1<<1)	; PLL Ready, read only 
GCR_DLL_EN				EQU (1<<16)	; DLL Enable 
GCR_DLL_RDY				EQU (1<<17)	; DLL Ready, read only 
GCR_DLL_SEL				EQU (1<<18)	; Select DLL phase 0 for DMC clock 
GCR_DLL_192M			EQU (1<<19)	; Select DLL 192Mhz (DDR400 devices supported)
GCR_PCKDIV				EQU 0x18	; Peripheral Clock Control Register 
GCR_PCKDIV_PCK			EQU 0x07	; Peripheral Clock Frequency Mask 
GCR_PCKDIV_OVR			EQU 0x08	; 0: Use dynamic value for PCKDIV 
GCR_PCKDIV_USB_SHIFT	EQU 4
GCR_PCKDIV_USB			EQU 0x70	; USB Clock Divider Mask 
GCR_PCKDIV_SDIO			EQU 0x280	; SDIO Clock Divider Mask 
GCR_PERCKCN				EQU 0x24	; Peripheral Clock Disable Register 
GCR_MEMCKCN				EQU 0x28	; Memory Clock Disable Register 
GCR_MEMZCN				EQU 0x2c	; Memory Zeroize Control Register 
GCR_MEMPD				EQU 0x30	; Memory Power Down Register 
GCR_SCCK				EQU 0x34	; Smart Card Clock Control Register 
GCR_MPRI0       		EQU 0x38	; AHB Master priority config 0 
GCR_MPRI1       		EQU 0x3c	; AHB Master priority config 1 

DMC_GCFG				EQU 0x00	; Global Config Register 
DMC_GCFG_BZERO			EQU (1<<0)
DMC_GCFG_PRCV0			EQU (1<<4)
DMC_GCFG_SRCV0			EQU (1<<5)
DMC_GCFG_PDRVCTL_SFT	EQU 0x08
DMC_GCFG_PCLKDLY_SFT	EQU 0x0b
DMC_GCFG_PCLKSEL		EQU (1<<14)
DMC_GCFG_PCLKDIS		EQU (1<<15)
DMC_GCFG_SDRVCTL_SFT	EQU 0x10
DMC_GCFG_SCLKDLY_SFT	EQU 0x12
DMC_GCFG_SCLKSEL		EQU (1<<22)
DMC_GCFG_SCLKDIS		EQU (1<<23)
DMC_GCFG_PADDRPD		EQU (1<<24)
DMC_GCFG_PADDRPU		EQU (1<<25)
DMC_GCFG_PDATPD			EQU (1<<26)
DMC_GCFG_PDATPU			EQU (1<<27)
DMC_GCFG_SADDRPD		EQU (1<<28)
DMC_GCFG_SADDRPU		EQU (1<<29)
DMC_GCFG_SDATPD			EQU (1<<30)
DMC_GCFG_SDATPU			EQU (1<<31)
DMC_CLK					EQU 0x04	; Clock Register 
DMC_STA					EQU 0x08	; Status Register 
DMC_STA_KEY0_VALID		EQU 0x01
DMC_STA_KEY1_VALID		EQU 0x02
DMC_STA_DLL_LOCKED		EQU 0x04
DMC_STA_ERR_PRIMARY		EQU 0x10000
DMC_STA_ERR_SECONDARY	EQU 0x20000
DMC_PCFG				EQU 0x80	; Primary SDRAM/DDR Configuration 
DMC_PCFG_TRFC_SFT		EQU 0x00
DMC_PCFG_TRCD_SFT		EQU 0x05
DMC_PCFG_TRP_SFT		EQU 0x08
DMC_PCFG_CASLAT_SFT		EQU 0x0c
DMC_PCFG_COLWIDTH_SFT	EQU 0x10
DMC_PCFG_ROWWIDTH_SFT	EQU 0x14
DMC_PCFG_ENCRYPT		EQU (1<<26)
DMC_PCFG_ENCRKEY		EQU (1<<27)
DMC_PCFG_INTEGRITY		EQU (1<<28)
DMC_PCFG_ENCERESP		EQU (1<<29)
DMC_PCFG_LPDDRSEL		EQU (1<<31)
DMC_PCMD				EQU 0x84	; Primary SDRAM Command Register 
DMC_PCMD_CMD_SFT		EQU 0x00
DMC_PCMD_EXECMD			EQU (1<<4)
DMC_PCMD_DISABLE		EQU (1<<8)
DMC_PCMD_ENABLE			EQU (1<<9)
DMC_PCMD_STATUS			EQU (1<<10)
DMC_PCMD_NORMAL			EQU 0x0
DMC_PCMD_NOP			EQU 0x1
DMC_PCMD_PRECHG			EQU 0x2
DMC_PCMD_LOADMODE		EQU 0x3
DMC_PCMD_SELFRFSH		EQU 0x4
DMC_PCMD_PWRDN			EQU 0x5
DMC_PCMD_AUTORFSH		EQU 0x6
DMC_PCMD_ZERO			EQU 0x7
DMC_PCMD_LOADEXMODE		EQU 0x8
DMC_PRFSH				EQU 0x88	; Primary SDRAM Refresh Register 
DMC_RFSH_ENCBOUND_MASK	EQU 0xffff0000
DMC_PAPD				EQU 0x8c	; Primary SDRAM Power-Down Register 
DMC_SCFG				EQU 0x90	; Secondary SDRAM Configuration 
DMC_SCMD				EQU 0x94	; Secondary SDRAM Command Register 
DMC_SRFSH				EQU 0x98	; Secondary SDRAM Refresh Register 
DMC_SAPD				EQU 0x9c	; Secondary SDRAM Power-Down Reg 
DMC_LDR_PEXT			EQU 0xa0	; Primary extended mode register 
DMC_LDR_PEXT_PASR_SFT	EQU 0x00
DMC_LDR_PEXT_TCSR_SFT	EQU 0x03
DMC_LDR_PEXT_DS_SFT		EQU 0x05
DMC_CFG0				EQU 0x100	; nCS[0] Config Register 
DMC_CFG1				EQU 0x110	; nCS[1] Config Register 
DMC_CFG2				EQU 0x120	; nCS[2] Config Register 
DMC_CFG3				EQU 0x130	; nCS[3] Config Register 
DMC_CFG4				EQU 0x140	; nCS[4] Config Register 
DMC_CFG5				EQU 0x150	; nCS[5] Config Register 
DMC_CFG6				EQU 0x160	; nCS[6] Config Register 
DMC_CFG7				EQU 0x170	; nCS[7] Config Register 
DMC_CFG_BUSW_MASK		EQU 0x03	; Data Bus Width 
DMC_CFG_BUSW_8			EQU 0x00
DMC_CFG_BUSW_16			EQU 0x01
DMC_CFG_BUSW_32			EQU 0x02
DMC_CFG_BCTRL			EQU 0x04
DMC_CFG_PAGEMD			EQU 0x20
DMC_CFG_PS				EQU 0x40
DMC_CFG_PGBOUND_MASK	EQU 0x700
DMC_CFG_PGBOUND_A2		EQU 0x000
DMC_CFG_PGBOUND_A3		EQU 0x100
DMC_CFG_PGBOUND_A4		EQU 0x200
DMC_CFG_PGBOUND_A5		EQU 0x300
DMC_CFG_PGBOUND_A6		EQU 0x400
DMC_CFG_PGBOUND_A7		EQU 0x500
DMC_CFG_ENCRYPT			EQU 0x1000
DMC_CFG_ENCRKEY			EQU 0x2000
DMC_CFG_INTEGRITY		EQU 0x4000
DMC_CFG_ENCERESP		EQU 0x8000
DMC_CFG_ENCBOUND_MASK	EQU 0xffff0000
DMC_RTIM0				EQU 0x104	; nCS[0] Read Timing Register 
DMC_RTIM1				EQU 0x114	; nCS[1] Read Timing Register 
DMC_RTIM2				EQU 0x124	; nCS[2] Read Timing Register 
DMC_RTIM3				EQU 0x134	; nCS[3] Read Timing Register 
DMC_RTIM4				EQU 0x144	; nCS[4] Read Timing Register 
DMC_RTIM5				EQU 0x154	; nCS[5] Read Timing Register 
DMC_RTIM6				EQU 0x164	; nCS[6] Read Timing Register 
DMC_RTIM7				EQU 0x174	; nCS[7] Read Timing Register 
DMC_RTIM_NRS_MASK		EQU 0x7f	; Read Cycle Setup Time 
DMC_RTIM_NRA_MASK		EQU 0xf0	; Read Access Time 
DMC_RTIM_NRH_MASK		EQU 0x700	; Read Cycle Hold Time 
DMC_RTIM_NRPA_MASK		EQU 0xf000	; Read Access Time Sequential 
DMC_WTIM0				EQU 0x108	; nCS[0] Write Timing Register 
DMC_WTIM1				EQU 0x118	; nCS[1] Write Timing Register 
DMC_WTIM2				EQU 0x128	; nCS[2] Write Timing Register 
DMC_WTIM3				EQU 0x138	; nCS[3] Write Timing Register 
DMC_WTIM4				EQU 0x148	; nCS[4] Write Timing Register 
DMC_WTIM5				EQU 0x158	; nCS[5] Write Timing Register 
DMC_WTIM6				EQU 0x168	; nCS[6] Write Timing Register 
DMC_WTIM7				EQU 0x178	; nCS[7] Write Timing Register 
DMC_WTIM_NWS_MASK		EQU 0x7f	; Write Cycle Setup Time 
DMC_WTIM_NWA_MASK		EQU 0xf0	; Write Access Time 
DMC_WTIM_NWH_MASK		EQU 0x700	; Write Cycle Hold Time 
DMC_WTIM_NRWI_MASK		EQU 0xf000	; Read to Write Time  
DMC_KEY0				EQU 0x200	; Encrypt Key 0 
DMC_KEY1				EQU 0x210	; Encrypt Key 1 

SIR_CFG					EQU 0x00	; Configuration Register 
SIR_CFG_SCRMBLD			EQU 0x20	; Scramble Bus Disable 
SIR_CFG_ENGMDEN			EQU 0x01

NAND_CFG				EQU 0x00	; Configuration Register 
NAND_CFG_BUSW_8			EQU 0x00
NAND_CFG_BCTRL			EQU 0x04
NAND_CFG_PAGEMD			EQU 0x20
NAND_RTIM				EQU 0x04	; Read Timing Register 
NAND_RTIM_NRS_MASK		EQU 0x7f	; Read Cycle Setup Time 
NAND_RTIM_NRA_MASK		EQU 0xf0	; Read Access Time 
NAND_RTIM_NRH_MASK		EQU 0x700	; Read Cycle Hold Time 
NAND_RTIM_NRPA_MASK		EQU 0xf000	; Read Access Time Sequential 
NAND_WTIM				EQU 0x08	; Write Timing Register 
NAND_WTIM_NWS_MASK		EQU 0x7f	; Write Cycle Setup Time 
NAND_WTIM_NWA_MASK		EQU 0xf0	; Write Access Time 
NAND_WTIM_NWH_MASK		EQU 0x700	; Write Cycle Hold Time 
NAND_WTIM_NRWI_MASK		EQU 0xf000	; Read to Write Time  
NAND_ECC_CTRL			EQU 0x0c	; ECC Control Register 
NAND_ECC_CTRL_EN		EQU 0x0001
NAND_ECC_CTRL_RST		EQU 0x0002
NAND_ECC_CTRL_MODE		EQU 0x0004
NAND_ECC_CTRL_WRITE		EQU 0x0008
NAND_ECC_STAT			EQU 0x10	; ECC Status Register 
NAND_ECC_STAT_BUSY		EQU 0x0001
NAND_ECC_STAT_EMPTY		EQU 0x0002
NAND_ECC_STAT_FULL		EQU 0x0004
NAND_ECC_STAT_NANDBUSY	EQU 0x0100
NAND_ECC_VAL			EQU 0x14	; ECC Value Register 

UCI_CR					EQU 0x000	; UCI Control Register 
UCI_CR_KACT_MASK		EQU 0x07
UCI_CR_KACT_ERASE		EQU 0x03
UCI_CR_KACT_LOAD		EQU 0x06
UCI_CR_KSRC_DIRECT		EQU 0x00
UCI_CR_KSRC_TRNG		EQU 0x08
UCI_CR_KSRC_OTP			EQU 0x10
UCI_CR_KSRC_SNVSRAM		EQU 0x18
UCI_CR_KID_SHIFT		EQU 8
UCI_CR_CLKDIV			EQU 0x1000
UCI_CR_IFLUSH			EQU 0x010000
UCI_CR_SCREN			EQU 0x020000
UCI_CR_KLOCK0			EQU 0x10000000
UCI_CR_KLOCK1			EQU 0x20000000
UCI_CR_KLOCK2			EQU 0x40000000
UCI_CR_KLOCK3			EQU 0x80000000
UCI_SR					EQU 0x004	; UCI Status register 
UCI_SR_ACDN				EQU 0x01	; Action done flag 
UCI_AC0					EQU 0x100	; UCI Area Config Register 0 
UCI_AC1					EQU 0x108	; UCI Area Config Register 1 
UCI_AC2					EQU 0x110	; UCI Area Config Register 2 
UCI_AC3					EQU 0x118	; UCI Area Config Register 3 
UCI_AC4					EQU 0x120	; UCI Area Config Register 4 
UCI_AC5					EQU 0x128	; UCI Area Config Register 5 
UCI_AC_KID_0			EQU 0x00
UCI_AC_KID_1			EQU 0x01
UCI_AC_KID_2			EQU 0x02
UCI_AC_KID_3			EQU 0x03
UCI_AC_OFF_MASK			EQU 0xfffff000
UCI_KEY0				EQU 0x200	; UCI AES Encryption Key 0 
UCI_KEY1				EQU 0x220	; UCI AES Encryption Key 1 
UCI_KEY2				EQU 0x240	; UCI AES Encryption Key 2 
UCI_KEY3				EQU 0x260	; UCI AES Encryption Key 3 
UCI_IKEY				EQU 0x2c0	; UCI Integrity Key 
UCI_SMR0				EQU 0x300	; UCI Static Memory Region 0 
UCI_SMR1				EQU 0x304	; UCI Static Memory Region 1 
UCI_SMR2				EQU 0x308	; UCI Static Memory Region 2 
UCI_SMR3				EQU 0x30c	; UCI Static Memory Region 3 
UCI_DDR0				EQU 0x310	; UCI DDR Region 0 
UCI_DDR1				EQU 0x314	; UCI DDR Region 1 
UCI_DDR2				EQU 0x318	; UCI DDR Region 2 
UCI_DDR3				EQU 0x31c	; UCI DDR Region 3 
UCI_NAND0				EQU 0x320	; UCI0 NAND Region 0 
UCI_NAND1				EQU 0x324	; UCI0 NAND Region 1 
UCI_NAND2				EQU 0x328	; UCI0 NAND Region 2 
UCI_NAND3				EQU 0x32c	; UCI0 NAND Region 3 
UCI_REGION_SHIFT		EQU 16
UCI_REGION_P			EQU (1 << 29)
UCI_REGION_E			EQU (2 << 29)
UCI_REGION_I			EQU (4 << 29)
UCI_REGION_EI			EQU (7 << 29)

UCI_ENCRYPTION_KEY	    EQU (UCI_AC_KID_0)




;----------------------- CODE --------------------------------------------------

                PRESERVE8



; Area Definition and Entry Point
;  Startup Code must be linked first at Address at which it expects to run.

                AREA    RESET, CODE, READONLY
                ARM

                IF      :LNOT::DEF:__EVAL 
                IF      :DEF:SIZE_INT_INFO
                IMPORT  ||Image$$ER_IROM1$$RO$$Length||
                IMPORT  ||Image$$RW_IRAM1$$RW$$Length||
                ELIF    :DEF:SIZE_EXT_INFO
                IMPORT  ||Image$$ER_ROM1$$RO$$Length||
                IMPORT  ||Image$$RW_RAM1$$RW$$Length||
                ENDIF
                ENDIF

; Exception Vectors
;  Mapped to Address 0.
;  Absolute addressing mode must be used.
;  Dummy Handlers are implemented as infinite loops which can be modified.

Vectors         LDR     PC,Reset_Addr         
                LDR     PC,Undef_Addr
                LDR     PC,SWI_Addr
                LDR     PC,PAbt_Addr
                LDR     PC,DAbt_Addr
                ; Reserved vector is used for image size information
                IF      :DEF:__EVAL
                  DCD   0x4000
                ELSE 
                  IF    :DEF:SIZE_INT_INFO
                    DCD ||Image$$ER_IROM1$$RO$$Length||+\
                        ||Image$$RW_IRAM1$$RW$$Length||
                  ELIF  :DEF:SIZE_EXT_INFO
                    DCD ||Image$$ER_ROM1$$RO$$Length||+\
                        ||Image$$RW_RAM1$$RW$$Length||
                  ELSE
                    NOP
                  ENDIF
                ENDIF
                LDR     PC,IRQ_Addr     
                LDR     PC,FIQ_Addr

;                IMPORT  IRQ_Handler


Reset_Addr      DCD     Reset_Handler
Undef_Addr      DCD     Undef_Handler
SWI_Addr        DCD     SWI_Handler
PAbt_Addr       DCD     PAbt_Handler
DAbt_Addr       DCD     DAbt_Handler
                DCD     0               ; Reserved Address
IRQ_Addr        DCD     IRQ_Handler
FIQ_Addr        DCD     FIQ_Handler

Undef_Handler   B       Undef_Handler
SWI_Handler     B       SWI_Handler
PAbt_Handler    B       PAbt_Handler
DAbt_Handler    B       DAbt_Handler
IRQ_Handler     B       IRQ_Handler
FIQ_Handler     B       FIQ_Handler


;IRQ_Handler
;		SUB	LR, LR,	#4
;		STMFD	SP!, {R0-R12,LR}
;		LDR	R6, =0xFFE03F00      ; w tym rejestrze znajdzie sie adres konkretnego przerwania (zaladowany automatycznie przez cpu
;		LDR	R5, [R6]
;		MOV	LR, PC
;		BX	R5
;		LDR	R0, =0xFFE03F08      ; do tego rejestru trzeba zapisac cokolwiek aby zakonczyc przerwanie
;		STR	R0, [R0]
;		LDMFD	SP!, {R0-R12,PC}^
  

; Reset Handler
                EXPORT  Reset_Handler
Reset_Handler   

    ; Ensure Watchdog is disabled */
	ldr	r0, =MAXIMASP_WDT_PHYS
	mov	r1, #0
	str	r1, [r0, #WDT_CTRL]

	 ; FLORA fixes:
	 ; when FLORA launches the user application (U-Boot),
	 ; some features remain enabled

	ldr	r0, =MAXIMASP_GCR_PHYS
	; Reset all GPIO pins to default state 
	mov	r1, #4			; bit 2 (RSTR.GPIO) 
	str	r1, [r0, #GCR_RSTR]
l1	ldr	r1, [r0, #GCR_RSTR]
	cmp	r1, #0
	bne	l1

	; disable the PLL and the DLL 
	ldr	r1, [r0, #GCR_CLKCN]
	; clear CLKSEL bits => Use crypto oscillator 
	and	r1, #(~GCR_CLKCN_CLKSEL_MASK)
	str	r1, [r0, #GCR_CLKCN]

	ldr	r1, [r0, #GCR_PLLCN]
	; clear PLLEN and DLLEN bits => Disable PLL and DLL 
	bic	r1, r1, #GCR_PLL_EN
	bic	r1, r1, #GCR_DLL_EN
	str	r1, [r0, #GCR_PLLCN]

	; Disable all interrupt sources 
	ldr	r0, =MAXIMASP_INTC_PHYS
	ldr	r1, =0xFFFFFFFF
	str	r1, [r0, #0x08]	; clear interrupt sources  0..31 
	str	r1, [r0, #0x14]	; clear interrupt sources 32..63 

	; enable all the peripheral clock sources 
	; (FLORA bootloader which disables [at least] the uart..)
	ldr	r0, =MAXIMASP_GCR_PHYS
	mov	r1, #0
	str	r1, [r0, #GCR_PERCKCN]

	; DDR clock trim overrides 
	; Set up GCR 
	ldr	r1, [r0, #GCR_SCON]
	bic	r1, #GCR_SCON_ADK
	bic	r1, #GCR_SCON_PBEN
	bic	r1, #GCR_SCON_UBP
	str	r1, [r0, #GCR_SCON]

	; Shadow register 4, DDR clock trim for B1/B2 
	; ldr	r0, =0xFFE00210
	; ldr	r1, =0x00001824
	; str r1, [r0, #0x0]

	; PLL trim override 
	ldr	r0, =0xFFE00100
	ldr	r1, [r0, #0x10]
	and	r1, #0xFFFFF0FF
	orr	r1, #0x00000800
	str	r1, [r0, #0x10]
	str	r1, [r0, #0x10]
	str	r1, [r0, #0x10]

	ldr	r0, =MAXIMASP_GCR_PHYS
	; Enable PLL 
	ldr	r1, [r0, #GCR_PLLCN]
	orr	r1, #GCR_PLL_EN
	str	r1, [r0, #GCR_PLLCN]

	; Wait for PLL lock 
l2 	ldr	r1, [r0, #GCR_PLLCN]
	tst	r1, #GCR_PLL_RDY
	beq	l2

	; Wait for at least 200uS. 250uS in this case. 
	ldr	r1, =(CONFIG_SYS_CLOCK/4000)>>2
l3	subs	r1, r1, #1
	bne	l3

	; JIBE B4/FLORA 3.2.2 exits with HDIV = /4
	; Set HDIV = /2
	; Set PDIV = /2 for good measure

	ldr	r1, [r0, #GCR_CLKCN]
	and	r1, #0xFFFFFFC0
	orr	r1, #0x00000009
	str	r1, [r0, #GCR_CLKCN]

	; Select PLL as clock 
	ldr	r1, [r0, #GCR_CLKCN]
	orr	r1, #(1 << GCR_CLKCN_CLKSEL_SHIFT)
	str	r1, [r0, #GCR_CLKCN]

	; Wait for at least 200uS. 250uS in this case. 
	ldr	r1, =(CONFIG_SYS_CPU_CLOCK/4000)>>2
l4	subs	r1, r1, #1
	bne	l4


	; Now running at 384MHz 

	; Enable DLL 
	ldr	r1, [r0, #GCR_PLLCN]
	orr	r1, #GCR_DLL_EN
	orr	r1, #GCR_DLL_192M	; on Bx 
	str	r1, [r0, #GCR_PLLCN]

	;  Wait for DLL lock 
l5	ldr	r1, [r0, #GCR_PLLCN]
	tst	r1, #GCR_DLL_RDY
	beq	l5

	; Wait for at least 200uS. 250uS in this case. 
	ldr	r1, =(CONFIG_SYS_CPU_CLOCK/4000)>>2
l6	subs	r1, r1, #1
	bne	l6

	; Set up GCR 
	ldr	r0, =MAXIMASP_GCR_PHYS
	ldr	r1, [r0, #GCR_SCON]
	orr	r1, #GCR_SCON_SYNC
	str	r1, [r0, #GCR_SCON]

	; Setup DMC 
	ldr	r0, =MAXIMASP_DMC_PHYS
	
	ldr	r1, =0x00004131     ; socketed B1 on EEKv1.3.0 

	ldr	r1, =0x05804000		; B4 enable pull down on data & addr */ 
	orr	r1, #(4<<8)         ; B4 primary drive control : half range slew rate */
	orr	r1, #(4<<11)        ; B4 primary clock delay : half range slew rate */

;	ldr	r1, =0x00854030     ; 1.4
;	ldr	r1, =0x04854000     ; 2.1
;	ldr	r1, =0x04004131     ; 2.3
;	ldr	r1, =0x04854030     ; 2.4

	str	r1, [r0, #DMC_GCFG]

    ; LP-DDR MT46H64M16LFBF-5 Configuration
	; 10 bits for COL
	; 14 bits for ROW
	; CAS Latency = 3
	; 16 bits mode
	; Bank address derived from MSBs (sequential mode)
	; tRFC =  72 ns min (14 cycles@192MHz)
	; tRCD =  15 ns min (3 cycles@192MHz)
	; tRP  =  15 ns min (3 cylces@192MHz)
	; Cycle time: 5ns(min) @ CL=3
	ldr	r1, =0x8032324D
;   ldr	r1, =0x803237FF	 ; 2.0
;	ldr	r1, =0x803337FF  ; 2.1
	str	r1, [r0, #DMC_PCFG]

	; stop refresh during the LP-DDR configuration 
	ldr	r1, =0x00000000
	str	r1, [r0, #DMC_PRFSH]

	; set Full LP-DDR strength mode 
	ldr	r1, =0x00000000
	str	r1, [r0, #DMC_LDR_PEXT]


	ldr	r1, =0x00000200
	str	r1, [r0, #DMC_PCMD]


	; Wait for at least 200uS. 250uS in this case. 
	ldr	r1, =(CONFIG_SYS_CPU_CLOCK/4000)>>2
l7	subs	r1, r1, #1	
	bne	l7

	; NOP 
	ldr	r1, =DMC_PCMD_NOP :OR: DMC_PCMD_STATUS :OR: DMC_PCMD_EXECMD
	str	r1, [r0, #DMC_PCMD]
lddr1
	ldr	r1, [r0, #DMC_PCMD]	; Wait for EXECMD bit to clear
	tst	r1, #DMC_PCMD_EXECMD
	bne	lddr1
	

	; Precharge 
	ldr	r1, =DMC_PCMD_PRECHG :OR: DMC_PCMD_STATUS :OR: DMC_PCMD_EXECMD
	str	r1, [r0, #DMC_PCMD]
lddr2
	ldr	r1, [r0, #DMC_PCMD]	; Wait for EXECMD bit to clear
	tst	r1, #DMC_PCMD_EXECMD
	bne	lddr2
	

	; NOP 
	ldr	r1, =DMC_PCMD_NOP :OR: DMC_PCMD_STATUS :OR: DMC_PCMD_EXECMD
	str	r1, [r0, #DMC_PCMD]
lddr3
	ldr	r1, [r0, #DMC_PCMD]	; Wait for EXECMD bit to clear
	tst	r1, #DMC_PCMD_EXECMD
	bne	lddr3


	; Auto refresh 
	ldr	r1, =DMC_PCMD_AUTORFSH :OR: DMC_PCMD_STATUS :OR: DMC_PCMD_EXECMD
	str	r1, [r0, #DMC_PCMD]
lddr4
	ldr	r1, [r0, #DMC_PCMD]	; Wait for EXECMD bit to clear
	tst	r1, #DMC_PCMD_EXECMD
	bne	lddr4


	; Auto refresh 
	ldr	r1, =DMC_PCMD_AUTORFSH :OR: DMC_PCMD_STATUS :OR: DMC_PCMD_EXECMD
	str	r1, [r0, #DMC_PCMD]
lddr5
	ldr	r1, [r0, #DMC_PCMD]	; Wait for EXECMD bit to clear
	tst	r1, #DMC_PCMD_EXECMD
	bne	lddr5

	; Load mode register 
	ldr	r1, =DMC_PCMD_LOADMODE :OR: DMC_PCMD_STATUS :OR: DMC_PCMD_EXECMD
	str	r1, [r0, #DMC_PCMD]
lddr6
	ldr	r1, [r0, #DMC_PCMD]	; Wait for EXECMD bit to clear
	tst	r1, #DMC_PCMD_EXECMD
	bne	lddr6

	; Load extended mode register 
	ldr	r1, =DMC_PCMD_LOADEXMODE :OR: DMC_PCMD_STATUS :OR: DMC_PCMD_EXECMD
	str	r1, [r0, #DMC_PCMD]
lddr7
	ldr	r1, [r0, #DMC_PCMD]	; Wait for EXECMD bit to clear
	tst	r1, #DMC_PCMD_EXECMD
	bne	lddr7


	; configure the refresh period 
	ldr	r1, =0x000001C2
	str	r1, [r0, #DMC_PRFSH]

	; Disable bus scrambling ???????????????????????????????????????????????
	ldr	r0, =MAXIMASP_SIR_PHYS
	ldr	r1, [r0, #SIR_CFG]
	orr	r1, #SIR_CFG_SCRMBLD
	str	r1, [r0, #SIR_CFG]

; #ifdef SIR_ENGDIS
	; Disable engineering mode
	ldr	r0, =MAXIMASP_SIR_PHYS
	ldr	r1, [r0, #SIR_CFG]
	bic	r1, #SIR_CFG_ENGMDEN
	str	r1, [r0, #SIR_CFG]

; #ifdef PCK_SETUP
	; Set up PCKDIV 
	ldr	r0, =MAXIMASP_GCR_PHYS
	mov	r3, #PCK_SETUP
	str	r3, [r0, #GCR_PCKDIV]


	; Initialize Static memory timing on CE0 and CE1 
	ldr	r0, =MAXIMASP_DMC_PHYS
	
	; Enable page mode access on CS5 
	; MT45W8MW16BGX, 8M x 16 
	; PSRAM page boundary is A[3:0], and are tied to JIBE A[4:1] 
	; Use A[5] page boundary for memory configuration setup. 
	ldr	r3, =(DMC_CFG_BCTRL :OR: DMC_CFG_BUSW_16 :OR: DMC_CFG_PAGEMD :OR: DMC_CFG_PGBOUND_A5)
	str	r3, [r0, #DMC_CFG5]
	; New PSRAM memory timings that are not dog slow: 72.92ns read, 72.92ns write, 20.83ns page read 
	ldr	r3, =0x03000409 ; Trpa: 20.83s, Trh: 0ns, Tra: 26.04ns, Trs: 46.88ns 
	str	r3, [r0, #DMC_RTIM5]
	ldr	r3, =0x00020803 ; Trwi: 5.2ns, Twh: 10.42ns, Twa: 46.88ns, Tws: 15.625ns 
	str	r3, [r0, #DMC_WTIM5]

	; Enable page mode access on CS6 
	; MT45W8MW16BGX, 8M x 16 
	; PSRAM page boundary is A[3:0], and are tied to JIBE A[4:1] 
	; Use A[5] page boundary for memory configuration setup. 
	ldr	r3, =(DMC_CFG_BCTRL :OR: DMC_CFG_BUSW_16 :OR: DMC_CFG_PAGEMD :OR: DMC_CFG_PGBOUND_A5)
	str	r3, [r0, #DMC_CFG6]
	; New PSRAM memory timings that are not dog slow: 72.92ns read, 72.92ns write, 20.83ns page read 
	ldr	r3, =0x03000409 ; Trpa: 20.83s, Trh: 0ns, Tra: 26.04ns, Trs: 46.87ns */
	str	r3, [r0, #DMC_RTIM6]
	ldr	r3, =0x00020803 ; Trwi: 5.2ns, Twh: 10.42ns, Twa: 46.88ns, Tws: 15.625ns */
	str	r3, [r0, #DMC_WTIM6]


	; Initialize CE4 timing for NOR Flash */
	mov	r3, #DMC_CFG_BUSW_16
	str	r3, [r0, #DMC_CFG4]
	ldr	r3, =0x0a0a0a0a
	str	r3, [r0, #DMC_RTIM4]
	ldr	r3, =0x0a0a0a0a
	str	r3, [r0, #DMC_WTIM4]

	; Initialize NAND Flash timing */
	ldr	r0, =MAXIMASP_NFC_PHYS
	mov	r3, #NAND_CFG_BUSW_8
	str	r3, [r0, #NAND_CFG]

	 ; these setting are for JIBEPOS (AHB of 192MHz), EEK
	 ; NAND Flash is faster than the JIBEPOS one, these settings
	 ; can be optimized.
	 ;
	 ; Read acces time 30ns/5.2ns  = 6 clk
	 ; Read hold time 25ns/5.2     = 5 clk
	 ; Read setup time 20ns/5.2    = 4 clk
	 ; Burst read access time 30ns = 6 clk
	 ; Write acces time 45ns/5.2ns = 9 clk
	 ; Write hold time 25ns/5.2    = 5 clk
	 ; Write setup time 20ns/5.2   = 4 clk
	 ; time after a read to a write 80ns/5.2 = 16 clk
	 
	ldr	r3, =0x06060504
	str	r3, [r0, #NAND_RTIM]
	ldr	r3, =0x0F050904
	str	r3, [r0, #NAND_WTIM]

	 ; Write RCR value in all PSRAM instances to enable page mode access.
	 ; NOTE: Make sure PSRAM memory accesses happen after timings are set up
	 ; as the PSRAM has a maximum CE low time of 4 microseconds and the reset
	 ; default CE low time is looking like 14 microseconds.
	 
psram_pagemodesetup
	; Set starting address/address of first PSRAM instance 
	ldr	r0, =0x44FFFFFE
	; Set address to last PSRAM instance 
	ldr	r2, =0x4BFFFFFE
	; Set address for NOR so we can break the CE access time down. 
	ldr	r4, =0x40000000
	; Loop through all 8 PSRAM instances 
psram_pagemodeloop
	; Set page mode 
	; Do two reads at maximum RAM address to unlock the PSRAM registers 
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 
	ldrh	r3, [r0]
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 
	ldrh	r3, [r0]
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 
	ldrh	r3, [r0]
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 
	ldrh	r3, [r0]
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 
	; Select RCR register 
	mov	r3, #0x00
	strh	r3, [r0]
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 
	; Enable Page Mode, Disable Deep Power Down Mode 
	mov	r3, #0x90
	strh	r3, [r0]
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 

	; Verify page mode was set 
	; Do two reads at maximum RAM address to unlock the PSRAM registers 
	ldrh	r3, [r0]
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 
	ldrh	r3, [r0]
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 
	; Select RCR register 
	mov	r3, #0x00
	strh	r3, [r0]
	ldrh	r3, [r4] ; Bogus memory op to cause nCS to release 
	; Verify that our RCR page mode settings were captured 
	ldrh	r3, [r0]
	cmp	r3,#0x90
	bne psram_pagemodeloop

	; Go to next 16MiB boundary 
	add	r0, #0x01000000
	cmp	r0, r2
	ble psram_pagemodeloop      ; Loop while r0 <= r2 */

	; Initialize all UCI Regions to Plain Text mode 
	ldr	r0, =MAXIMASP_UCI_0_PHYS

	 ; Note: the UCI region size for the Static memory is 10 bits,
	 ; but the DDR & NAND are 11 bits (see below).
	 ;
	 ; The maximum size per Static region is:
	 ;    (2^10)-1 * 64KiB  = 67 043 328 bytes (63+MiB < 64MiB)
	 ;
	 ; NOTE: The Static memories on Maxim's EEK are all on the
	 ;       Secondary Bus (Bus 1), which does not have an UCI
	 ;       (on B1/B2).  Hence, there is nothing to configure.
	 
	 ; Region size computation (for the NAND & DDR regions):
	 ;
	 ; bit 16..26 are the number of 64KiB chunks (numbered from 1 to N).
	 ; If you want 8MiB, the number of chunks is 8MiB/64KiB.
	 ; The number of chunks is coded on bits 16 to 26, so we must do
	 ; a 16 bits right shift, so the region length (number of chunks)
	 ; will be (8MiB>>16)<<16 = 8MiB
	 ;
	 ; NOTE: take care to the region size which is coded on 11 bits
	 ;       Knowing that, the maximum region size is
	 ;
	 ;         (2^11)-1 blocks ==> 2047 (0x7FF) blocks of 64KiB (0x10000)
	 ;                         ==> 134 152 192 bytes (127+MiB < 128MiB)
	 ;
	 ;       If you want to use a bigger memory, you must use several
	 ;       regions.
	 ;
	 ;	when using an integrity (I or E+I mode) region,
	 ;	the data and the integrity data are interleaved.
	 ;
	 ;           Core address space       external address space
	 ;core[0]->+--------------------+     +--------------------+ <- ext[0]
	 ;	   | Plain or Encr Only |     | Plain or Encr Only |
	 ;core[1]->+--------------------+     +--------------------+ <- ext[1]
	 ;	   |                    |\    |   Not AVAILABLE    |
	 ;	   |                    | \___+--------------------+ <- ext[2]
	 ;	   |                    |     |  User Data block   |
	 ;	   |                    |     +--------------------+
	 ;	   |   User Data block  |     |   Integrity Data   |
	 ;	   |    E + I mode OR   |     +--------------------+
	 ;	   |    Integrity only  |     |   ......           |
	 ;	   |       32 MiB       |     +--------------------+ <- ext[n]
	 ;	   |                    |     |  User Data block   |
	 ;core[2]->+--------------------+     +--------------------+ <- ext[n+1]
	 ;	   |   Not AVAILABLE    |     |   Integrity Data   |
	 ;core[3]->+--------------------+_____+--------------------+ <- ext[n+2]
	 ;	   |   Plain or E only  |     |   Plain or E only  |
	 ;	   +--------------------+_____+--------------------+ <- [Next Region]
	 ;
	 ;	use the following formulas to know the starting/ending addresses:
	 ;	- data address translated:
	 ;		addr_out = addr_in + (((addr_in - OFFSET)>>3) & 0xFFFFFFF0);
	 ;
	 ;	- integrity address translated:
	 ;		addr_out = ((addr_in - OFFSET) & 0xFFFFFF80) +
	 ;			0x0000 0080 + ((addr_in - OFFSET)>>3)+OFFSET;
	 ;
	 ;	Where OFFSET if the memory base address.
	 ;	Example:
	 ;		we have a first region of 16 MBytes plaintext starting
	 ;		at core[0] (OFFSET=0x70000000) and ending
	 ;		at core[1]-1 (0x70FFFFFF)
	 ;
	 ;		we want to declare a 32 MBytes E+I region after the
	 ;		region which ends at core[1]-1.
	 ;		As the new region uses Integrity, UCI will do the
	 ;		translation job for us.
	 ;		that means, for the core address space the E+I chunk
	 ;		starts at core[1] (which is 0x71000000 in the core space);
	 ;		On the external device address space the starting address
	 ;		will be translated using the 'data address translated',
	 ;		the ext[2] will be:
	 ;		ext[2] = 0x71000000 + (0x1000000>>3) & 0xFFFFFFFE
	 ;		ext[2] = 0x71200000
	 ;
	 ;		In the core space, the E+I region will end at
	 ;		offset core[2] = 48MiB.
	 ;
	 ;		If we want to allocate a new integrity region (I or E+I)
	 ;		after the E+I one, we can just use this offset (as UCI
	 ;		will automatically translate the addresses).
	 ;
	 ;		But if we want to use a non integrity block
	 ;		(E or Plaintext), we must compute the new starting
	 ;		address using the 'integrity address translated'
	 ;		forumula.
	 ;
	 ;		example: we add a Encryption only region after the E+I:
	 ;		we must compute the core[3] address:
	 ;		core[3] = (0x3000000 & 0xFFFFFF80) + (0x3000000>>3) +
	 ;			  0x80 + 0x70000000
	 ;		core[3] = 0x73600080
	 ;
	 ;		Our new Encryption Only region will start at 0x73600080.
	 
	 ;	Remember that if you use several Integrity regions (EI or I),
	 ;	the translation is done by the HW... you just have to enter the
	 ;	start offset and the length.
	 ;
	 ;	If you use several Integrity-less regions, there is no translation
	 ;	at all. If your intergrity-less region follow an intergrity
	 ;	region, you must compute the new starting address.
	 ;

    ldr	r1, =(UCI_REGION_P :OR: SZ_64M :OR: (MAXIMASP_NAND >> UCI_REGION_SHIFT))
	str	r1, [r0, #UCI_NAND0]
	add	r1, r1, #(SZ_64M >> UCI_REGION_SHIFT)
	str	r1, [r0, #UCI_NAND1]
	add	r1, r1, #(SZ_64M >> UCI_REGION_SHIFT)
	str	r1, [r0, #UCI_NAND2]
	add	r1, r1, #(SZ_64M >> UCI_REGION_SHIFT)
	str	r1, [r0, #UCI_NAND3]


; IF ENCRYPTION  (WITH ENCRYPTION) ----------------------------------------------------------------------------------------------------

	ldr	r1, =(UCI_REGION_E  :OR: (SZ_128M>>1) :OR: (MAXIMASP_DDR0 >> UCI_REGION_SHIFT))
	str	r1, [r0, #UCI_DDR0]

	ldr	r1, =(UCI_REGION_E  :OR: (SZ_128M>>1) :OR: (MAXIMASP_DDR0 + (SZ_128M>>1)) >> UCI_REGION_SHIFT)
	str	r1, [r0, #UCI_DDR1]


	ldr	r0, =MAXIMASP_UCI_0_PHYS
	; Configure integrity key 
	ldr	r1, =0xaa55aa55
	str	r1, [r0, #UCI_IKEY]
	str	r1, [r0, #(UCI_IKEY + 0x04)]
	str	r1, [r0, #(UCI_IKEY + 0x08)]
	str	r1, [r0, #(UCI_IKEY + 0x0c)]

	; Set up key register 0 
	ldr	r1, =0x12345678
	str	r1, [r0, #UCI_KEY0]
	str	r1, [r0, #(UCI_KEY0 + 0x04)]
	str	r1, [r0, #(UCI_KEY0 + 0x08)]
	str	r1, [r0, #(UCI_KEY0 + 0x0c)]

	; Set up key register 1 
	ldr	r1, =0x1ABCDEF1
	str	r1, [r0, #UCI_KEY1]
	str	r1, [r0, #(UCI_KEY1 + 0x04)]
	str	r1, [r0, #(UCI_KEY1 + 0x08)]
	str	r1, [r0, #(UCI_KEY1 + 0x0c)]

	; Set up key register 2 
	ldr	r1, =0xDEADBEEF
	str	r1, [r0, #UCI_KEY2]
	str	r1, [r0, #(UCI_KEY2 + 0x04)]
	str	r1, [r0, #(UCI_KEY2 + 0x08)]
	str	r1, [r0, #(UCI_KEY2 + 0x0c)]

	; Set up key register 3 
	ldr	r1, =0x7FEDCBA6
	str	r1, [r0, #UCI_KEY3]
	str	r1, [r0, #(UCI_KEY3 + 0x04)]
	str	r1, [r0, #(UCI_KEY3 + 0x08)]
	str	r1, [r0, #(UCI_KEY3 + 0x0c)]


	; Configure key regions 
	ldr	r1, =(((MAXIMASP_DDR0) & UCI_AC_OFF_MASK) :OR: UCI_ENCRYPTION_KEY)
	ldr	r2, =((MAXIMASP_DDR0 + SZ_128M) & UCI_AC_OFF_MASK)
	str	r1, [r0, #UCI_AC0]
	str	r2, [r0, #(UCI_AC0 + 0x04)]


	mov	r3, #0
	str	r3, [r0, #UCI_AC1]
	str	r3, [r0, #(UCI_AC1 + 0x04)]

	str	r3, [r0, #UCI_AC2]
	str	r3, [r0, #(UCI_AC2 + 0x04)]
	str	r3, [r0, #UCI_AC3]
	str	r3, [r0, #(UCI_AC3 + 0x04)]
	str	r3, [r0, #UCI_AC4]
	str	r3, [r0, #(UCI_AC4 + 0x04)]
	str	r3, [r0, #UCI_AC5]
	str	r3, [r0, #(UCI_AC5 + 0x04)]

	; Wait for UCI ready 
uc1 ldr	r1, [r0, #UCI_CR]
	tst	r1, #UCI_CR_KACT_MASK
	bne	uc1

	; Load decryption key 
	ldr	r1, =(UCI_CR_KSRC_DIRECT :OR: UCI_CR_IFLUSH :OR: (UCI_ENCRYPTION_KEY<<UCI_CR_KID_SHIFT))
	str	r1, [r0, #UCI_CR]
	orr     r1, #UCI_CR_KACT_LOAD
	str	r1, [r0, #UCI_CR]

	; Wait for UCI ready 
uc2	ldr	r1, [r0, #UCI_SR]
	tst	r1, #UCI_SR_ACDN
	beq	uc2
	
; ENDIF     (ENCRYPTION)  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^






; IF WITHOUT ENCRYPTION 
;	ldr	r1, =(UCI_REGION_P  :OR: (SZ_128M>>1) :OR: (MAXIMASP_DDR0 >> UCI_REGION_SHIFT))
;	str	r1, [r0, #UCI_DDR0]
;	ldr	r1, =(UCI_REGION_P  :OR: (SZ_128M>>1) :OR: (MAXIMASP_DDR0 + (SZ_128M>>1)) >> UCI_REGION_SHIFT)
;	str	r1, [r0, #UCI_DDR1]
;ENDIF WITHOUT ENCRYPTION 



; ---------------------------------------

                IF      :DEF:RAM_INTVEC
                ADR     R8,  Vectors    ; Source
                LDR     R9, =IRAM_BASE  ; Destination
                LDMIA   R8!, {R0-R7}    ; Load Vectors 
                STMIA   R9!, {R0-R7}    ; Store Vectors 
                LDMIA   R8!, {R0-R7}    ; Load Handler Addresses 
                STMIA   R9!, {R0-R7}    ; Store Handler Addresses
                ENDIF



; Vector Floating-Point (VFP) Setup --------------------------------------------

                IF      {FPU} != "SoftVFP"
                MOV     R1, #VFP_EN_BIT ; Enable VFP
                FMXR    FPEXC, R1
                ENDIF



; Setup Stack for each mode ----------------------------------------------------

                LDR     R0, =Stack_Top

;  Enter Undefined Instruction Mode and set its Stack Pointer
;                MSR     CPSR_c, #Mode_UND:OR:I_Bit:OR:F_Bit
;                MOV     SP, R0
;                SUB     R0, R0, #UND_Stack_Size

;  Enter Abort Mode and set its Stack Pointer
;                MSR     CPSR_c, #Mode_ABT:OR:I_Bit:OR:F_Bit
;                MOV     SP, R0
;                SUB     R0, R0, #ABT_Stack_Size

;  Enter FIQ Mode and set its Stack Pointer
;                MSR     CPSR_c, #Mode_FIQ:OR:I_Bit:OR:F_Bit
;                MOV     SP, R0
;                SUB     R0, R0, #FIQ_Stack_Size

                IMPORT  ||Image$$CSTACK$$ZI$$Limit||       ; defined in scatter file
                IMPORT  ||Image$$IRQ_STACK$$ZI$$Limit||    ; defined in scatter file

;  Enter IRQ Mode and set its Stack Pointer
                MSR     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit
                ldr     r0, =||Image$$IRQ_STACK$$ZI$$Limit||
                and     r0, r0, #0xFFFFFFF8
                MOV     SP, R0
;                SUB     R0, R0, #IRQ_Stack_Size

;  Enter Supervisor Mode and set its Stack Pointer
                MSR     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit
                ldr     r0, =||Image$$CSTACK$$ZI$$Limit||
                and     r0, r0, #0xFFFFFFF8
                MOV     SP, R0
;                SUB     R0, R0, #SVC_Stack_Size

;  Enter User Mode and set its Stack Pointer
;                MSR     CPSR_c, #Mode_USR
;                IF      :DEF:__MICROLIB

;                EXPORT __initial_sp

;                ELSE

;                MOV     SP, R0
;                SUB     SL, SP, #USR_Stack_Size

;                ENDIF

; call low level init ----------------------------------------------------------
                IMPORT  __low_level_init
                LDR     R0, =__low_level_init
                BLX     R0

; Enter the C code

                IMPORT  __main
                LDR     R0, =__main
                BX      R0

                IF      :DEF:__MICROLIB

                EXPORT  __heap_base
                EXPORT  __heap_limit

                ELSE
; User Initial Stack & Heap
                AREA    |.text|, CODE, READONLY

                IMPORT  __use_two_region_memory
                EXPORT  __xxuser_initial_stackheap
__xxuser_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + USR_Stack_Size)
                LDR     R2, = (Heap_Mem +      Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR
                ENDIF


                END
