/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2014  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.04a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : OS_Config.h
Purpose : Configuration settings for the OS
          Setup configuration according to your hardware
--------  END-OF-HEADER  ---------------------------------------------
*/

#ifndef OS_CONFIG_H                     /* Avoid multiple inclusion */
#define OS_CONFIG_H

/*********************************************************************
*
*       Clock frequency settings
*
*       The MCB2100 starter board is delivered with a
*       12.0 MHz crystal which is default CPU clock 
*       if PLL is not initialized
*/

#define _CRYSTAL_CLOCK 12000000L  /* external crystal frequency   */
#define OS_INIT_PLL 1             /* Init PLL during startup      */
#define OS_INIT_MAM 1             /* Init memory accelerator      */

/*********************************************************************
*
*       UART settings for embOSView
*/

#define OS_UART 1
#define OS_BAUDRATE 38400L

/****** End of modifyable configuration settings ********************/

/*********************************************************************
*
*       Calculated configuration settings
*/

#ifndef OS_INIT_PLL
  #define OS_INIT_PLL 0
#endif

#if OS_INIT_PLL
  #define OS_PLL_MULTIPLIER   4   /* PLL multiplies by 4          */
#else
  #define OS_PLL_MULTIPLIER   1   /* PLL not used                 */
#endif

#define OS_FSYS (_CRYSTAL_CLOCK * OS_PLL_MULTIPLIER)

/********************************************************************/

#endif                                  /* Avoid multiple inclusion */

/*****  EOF  ********************************************************/
