/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2014  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.04a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : LED interface (for LPC2378 and KEIL MCB2300 board)
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/
#define PINSEL10       (*(volatile unsigned int*) (0xE002C028))
#define FIO2DIR		   (*(volatile unsigned int*) (0x3FFFC040))
#define FIO2MASK	   (*(volatile unsigned int*) (0x3FFFC050))
#define FIO2PIN		   (*(volatile unsigned int*) (0x3FFFC054))
#define FIO2SET		   (*(volatile unsigned int*) (0x3FFFC058))
#define FIO2CLR		   (*(volatile unsigned int*) (0x3FFFC05C))

/****** Map LED port to CPU Port ************************************/

#define _LEDPORT_SET   (FIO2SET)
#define _LEDPORT_CLR   (FIO2CLR)
#define _LEDPORT_STATE (FIO2PIN)
#define _LED0_BIT      (0)
#define _LEDMASK_ALL   (0xFF)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

//#include <LPC23xx.h>                    /* LPC23xx definitions                */

/*********************************************************************
*
*       LED_Init()
*/

void BSP_Init(void) {
  PINSEL10 = 0;               /* Disable ETM interface, enable LEDs */
  FIO2DIR  = _LEDMASK_ALL;    /* P2.0..7 defined as Outputs         */
  FIO2MASK = 0x00000000;
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  _LEDPORT_SET = (1 << (_LED0_BIT + Index)); 
}

void BSP_ClrLED(int Index) {
  _LEDPORT_CLR = (1 << (_LED0_BIT + Index)); 
}

void BSP_ToggleLED(int Index) {
#if 0
  if (_LEDPORT_STATE & (1 << (_LED0_BIT + Index))) {  /* LED was on */
    BSP_ClrLED(Index);
  } else {
    BSP_SetLED(Index);
  }
#else
  _LEDPORT_STATE ^= (1 << (_LED0_BIT + Index));
#endif
}

/****** EOF *********************************************************/

